/*
 *  
 *  Copyright (c) 2004 Steve Slaven & Antti Andreimann, All Rights Reserved.
 *  
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of
 *  the License, or (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA 02111-1307 USA
 *  
*/

package net.hoopajoo.cheeselab3;

import javax.swing.*;        
import java.awt.event.*;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;
import net.hoopajoo.swingx.*;

/**
 * Drawing Toolbox for CheeseMap
 * @author anttix
 */
public class CheeseMapToolbox extends JToolBar {
	CheeseMapTool [] tools;
	
	CheeseMapToolbox() {
		super();
		
		JToggleButton tbb;
		JMenuItem item;
		
		// Create tools
		tools = new CheeseMapTool[4];
		tools[0] = new CheeseMapTool ( new ImageIcon ("ui/ui_select.png"),
		        						"select_tool",	"Select Tool", this );
		tools[1] = new CheeseMapTool ( new ImageIcon ("ui/ui_merge.png"),
		        						"merge_tool",	"Merge Tool", this );
		tools[2] = new CheeseMapTool ( new ImageIcon ("ui/ui_move.png"),
										"move_tool",	"Move Tool", this );
		tools[3] = new CheeseMapTool ( new ImageIcon ("ui/ui_delete.png"),
										"delete_tool",	"Delete Tool", this );

		// add buttons
		for( int i = 0; i < tools.length; i++ ) {
		    tbb = new JSelectedButton ( tools[i] );
		    add( tbb );
		}
		
		tools[0].setSelected( true );		
	}
	
	/**
	 * Go through all toolbox buttons and deselect them.
	 */
	
	class CheeseMapTool extends SelectedAction {
	    CheeseMapToolbox toolbox;
	    
	    public CheeseMapTool(ImageIcon icon, String name, String desc, CheeseMapToolbox toolbox) {
	        super();
	        putValue( ACTION_COMMAND_KEY, name );
	        putValue( SHORT_DESCRIPTION, desc );
	        putValue( SMALL_ICON, icon );
	        this.toolbox = toolbox;
	    }
	    public void actionPerformed(ActionEvent e) {
	    	String tool = e.getActionCommand();
	    	toolbox.setActiveTool( tool );
	    }
	    public String getName() {
	        return (String)getValue( ACTION_COMMAND_KEY );
	    }
	}
		
	class CheeseMapToolMenuItem extends JSelectedRadioMenuItem {
	    CheeseMapToolMenuItem(CheeseMapTool a) {
	        super( a );
	    }
	    protected void configurePropertiesFromAction(Action a) {
	        super.configurePropertiesFromAction( a );
	        setIcon( null );
	    }
	    protected PropertyChangeListener createActionPropertyChangeListener(Action a) {
	        a.addPropertyChangeListener( super.createActionPropertyChangeListener ( a ));
	        
	        return new PropertyChangeListener() {
	            public void propertyChange(PropertyChangeEvent evt) {
	                if ( evt.getPropertyName().equals( CheeseMapTool.SMALL_ICON ) ) {
	                    setIcon( null );
	                }
	            }
	        };
	    }
	}
	
	/**
	 * Provide name of the currently active tool
	 * @return  The name of the active tool, null if none active (shouldn't happen)
	 */
	public String getActiveTool() {
	    for (int i = 0; i < tools.length; i++) {
	        if( tools[i].isEnabled() )
	            return tools[i].getName();
	    }
		return null;
	}
	
	/**
	 * Provide programmatic control to activate a specific tool
	 * @param toolname  a tool name to activate (see class constructor for list)
	 */
	public void setActiveTool( String toolname ) {
	    for (int i = 0; i < tools.length; i++) {
	        if( tools[i].isEnabled() )
	            if ( toolname.equals( tools[i].getName() ) )
	                tools[i].setSelected( true );
	            else
	                tools[i].setSelected( false );
	    }

		CheeseMap.set_edit_tool( toolname );
	}
	
	/**
	 * Create a menu that has a list of all available tools in it. 
	 * @param label  A label to be used for new menu
	 * @return a menu instance
	 */
	public JMenu createMenu(String label) {
		JMenu submenu = new JMenu( label );
		 
		// Create buttongroup for tool selection access inside menus
		ButtonGroup toolbg = new ButtonGroup();
		
		for( int i = 0; i < tools.length; i++ ) {
		    CheeseMapToolMenuItem item = new CheeseMapToolMenuItem( tools[i] );
		    toolbg.add( item );
		    submenu.add( item );
		}
				
		return submenu;
	}
}